from qt import *
##~ from photolibrary import LPhotoLibrary
from utils import *

import os, errno
import shutil


class LImportItemEvent(QCustomEvent):
	def __init__(self, key, dict, progress, total, albumPath=None):
		QCustomEvent.__init__(self, QEvent.User+1)
		self.setData(dict)
		self.key = key
		self.dict = dict
		self.progress = progress
		self.total = total
		self.albumPath = albumPath
		
	def getKey(self):
		return self.key
		
	def getDict(self):
		return self.dict
		
	def getAlbumPath(self):
		return self.albumPath
		
	def getProgress(self):
		return (self.progress, self.total)

class LImportCompleteEvent(QCustomEvent):
	def __init__(self, source):
		QCustomEvent.__init__(self, QEvent.User+2)
		self.source = source
		
	def getSource(self):
		return self.source

class LImportThread(QThread):
	def __init__(self, target):
		QThread.__init__(self)
		self.target = target
		self._validExtensions = [".jpg",".jpeg"]
		
	def setImportObject(self, source):
		self.source = source
		
	def __findNewFilename(self, path, filename):
		(a,b) = os.path.splitext(filename)
		newName = a + b
		check = str(os.path.join(path, newName))
		i = 1
		while (LUtils.fileExists(check)):
			newName = a + str(i) + b
			check = os.path.join(path, newName)
			i = i + 1
		return newName
		
	def run(self):
		from photolibrary import LPhotoLibrary
		from utils import LUtils
		import time
		
		files = self.source.getFileList()
		print "import thread file count=", len(files)
		i = 0
		total = len(files)
		# send zero event to init progress bar
		QApplication.postEvent(self.target, LImportItemEvent(None, None, i, total))
		for file in files:
			i = i + 1
			print file
			tmpPath = self.source.copyToFilesystem(file)
			filename = self.source.getFilename(file)
			albumPath = self.source.getInsertAlbumPath(file)
			#(path, filename) = os.path.split(str(file))
			
			#build dictionary of image propertiesAlbum 7
			(mode,ino,dev,nlink,uid,gid,size,atime,mtime,ctime) = os.stat(tmpPath)
			importDestPath = str(os.path.join(LPhotoLibrary.libraryPath(), LUtils.datePath(mtime)))
			LUtils.ensureDirectory(importDestPath)
			thumbPath = os.path.join(importDestPath, "Thumbs")
			LUtils.ensureDirectory(thumbPath)
			photo = {}
			photo['Date'] = mtime
			photo['ImportDate'] = int(time.time())
			photo['Caption'] = filename
			photo['Rating'] = 0
			photo['Comments'] = ""
			
			proposedPath = str(os.path.join(importDestPath, filename))
			key = filename
			
			#copy file into library
			if LUtils.fileExists(proposedPath):
				# check if file is the same size
				if os.path.getmtime(str(tmpPath)) == os.path.getmtime(proposedPath):
					# make sure this image is in the master library
					self.source.cleanupFilesystem(tmpPath)
					QApplication.postEvent(self.target, LImportItemEvent(None, None, i, total))
					continue
				# filename is in use, but image is not a copy
				filename = self.__findNewFilename(importDestPath, filename)
				
			photo['ImagePath'] = str(os.path.join(importDestPath, filename))
			photo['ThumbPath'] = str(os.path.join(thumbPath, filename))
				
			self.source.copyToLibrary(tmpPath, photo['ImagePath'])
			
			#create thumbnail
			(w,h) = LUtils.downsizeImage(photo['ImagePath'], photo['ThumbPath'])
			if h==0 or w==0:
				continue # no width or height
				
			photo['AspectRatio'] = w/float(h)#LUtils.createThumbnail(photo['ImagePath'], photo['ThumbPath'])
			photo['Width'] = w
			photo['Height'] = h
			QApplication.postEvent(self.target, LImportItemEvent(key, photo, i, total, albumPath))
			
		#photo dict is complete, send back to library
		QApplication.postEvent(self.target, LImportCompleteEvent(self.source))
